<?php
/**
 * @package    JMyLife
 * @subpackage Components
 * @version    1.0.16
 * @copyright  2009-2010 Jeff Channell
 * administrator/components/com_jmylife/helpers/image.php
 * @license

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 * @since     1.0.14
 */
// no direct access
defined('_JEXEC') or die('Restricted access');
jimport('joomla.filesystem.file');
if (!class_exists('JmylifeConfiguration')) {
	$configfile = JPATH_ROOT.DS.'components'.DS.'com_jmylife'.DS.'libraries'.DS.'configuration.php';
	if( JFile::exists( $configfile ) ) require_once( $configfile );
}

class JmylifeImageHelper
{
	private static $_instance;
	var $cfg;
	private $mimes = array(
		'image/jpeg' => 'jpg',
		'image/pjpeg' => 'jpg',
		'image/png' => 'png',
		'image/x-png' => 'png',
		'image/gif' => 'gif'
	);

	function __construct() {
		$this->cfg = JmylifeConfiguration::getInstance();
	}

	function getInstance() {
		if (!self::$_instance) {
			self::$_instance = new JmylifeImageHelper();
		}
		return self::$_instance;
	}

	/**
	 * validate
	 * @param   string
	 * @return  mixed    true extension if valid, false if not
	 */
	function validate($file) {
		if (!JFile::exists($file)) {
			return false;
		}
		// dimensions
		$imageinfo = @getimagesize($file);
		if (!$imageinfo) {
			return false;
		}
		if (!is_int($imageinfo[0])) {
			return false;
		}
		if (!is_int($imageinfo[1])) {
			return false;
		}
		if (!in_array($imageinfo['mime'], array_keys($this->mimes))) {
			return false;
		}
		return $this->mimes[$imageinfo['mime']];
	}

	/**
	 * function resize
	 * @param   string
	 * @param   string
	 * @param   int
	 * @param   int
	 * @return  bool
	 */
	function resize($file, $ext, $width, $height) {
		if ('png' == $ext) {
			$src = imagecreatefrompng($file);
		}
		else if ('jpg' == $ext) {
			$src = imagecreatefromjpeg($file);
		}
		else {
			return false;
		}
		// get image dimensions
		$x = imageSX($src);
		$y = imageSY($src);
		// set image dimensions to width & height if image is small
		if ($x < $width && $y < $height) {
			$width = $x;
			$height = $y;
		}
		if ($x > $y) {
			$w = $width;
			$h = $y * ($width / $x);
		}
		if ($x < $y) {
			$w = $x * ($height / $y);
			$h = $height;
		}
		if ($x == $y) {
			$w = $width;
			$h = $height;
		}
		$img = ImageCreateTrueColor($w, $h);
		if ('png' == $ext) {
			// pngs need their alpha preserved
			imagealphablending($img, false);
			imagesavealpha($img, true);
		}
		imagecopyresampled($img, $src, 0, 0, 0, 0, $w, $h, $x, $y);
		if ('png' == $ext) {
			imagepng($img, $file);
		} else {
			imagejpeg($img, $file);
		}
		imagedestroy($img);
		imagedestroy($src);
		return true;
	}

}