<?php
/**
 * @package    JMyLife
 * @subpackage Components
 * @version    1.0.16
 * @copyright  2009-2010 Jeff Channell
 * administrator/components/com_jmylife/helpers/story.php
 * @license

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 * @since     1.0.14
 */
// no direct access
defined('_JEXEC') or die('Restricted access');
jimport('joomla.filesystem.file');
if (!class_exists('JmylifeConfiguration')) {
	$configfile = JPATH_ROOT.DS.'components'.DS.'com_jmylife'.DS.'libraries'.DS.'configuration.php';
	if( JFile::exists( $configfile ) ) require_once( $configfile );
}
if (!class_exists('JmylifeCommon')) {
	$commonfile = JPATH_ROOT.DS.'components'.DS.'com_jmylife'.DS.'libraries'.DS.'common.php';
	if( JFile::exists( $commonfile ) ) require_once( $commonfile );
}

class JmylifeStoryHelper
{
	private static $_instance;
	private $_cache;
	var $cfg;
	var $columns;
	var $db;
	var $html;
	var $itemid;
	var $str;
	var $url;

	function __construct() {
		JLoader::import('components.com_jmylife.helpers.database', JPATH_ADMINISTRATOR, '');
		JLoader::import('components.com_jmylife.helpers.html', JPATH_ADMINISTRATOR, '');
		JLoader::import('components.com_jmylife.helpers.itemid', JPATH_ADMINISTRATOR, '');
		JLoader::import('components.com_jmylife.helpers.string', JPATH_ADMINISTRATOR, '');
		JLoader::import('components.com_jmylife.helpers.url', JPATH_ADMINISTRATOR, '');
		$this->_cache = array();
		$this->cfg = JmylifeConfiguration::getInstance();
		$this->db = JmylifeDatabaseHelper::getInstance();
		$this->html = JmylifeHtmlHelper::getInstance();
		$this->itemid = JmylifeItemidHelper::getInstance();
		$this->str = JmylifeStringHelper::getInstance();
		$this->url = JmylifeUrlHelper::getInstance();
		$this->columns = array('id', 'catid', 'time', 'fulltext', 'published', 'votes1', 'votes2', 'reported', 'moderate1', 'moderate2', 'via', 'user_id', 'user_alias', 'user_email', 'allow_votes', 'allow_comments', 'admin_fulltext', 'title');
	}

	function getInstance() {
		if (!self::$_instance) {
			self::$_instance = new JmylifeStoryHelper();
		}
		return self::$_instance;
	}

	/**
	 * quickLoad
	 * @param  int     id of story
	 * @param  mixed   columns to load
	 * @return object
	 *
	 * used to quickload a story
	 * $columns expected to be a string or array
	 */
	function quickLoad($id, $columns=false) {
		$id = max(0, intval($id));
		$cols = array();
		if (!is_array($columns)) {
			$columns = array($columns);
		}
		if (count($columns)) {
			foreach ($columns as $c) {
				if (in_array($c, $this->columns)) {
					$cols[] = $this->db->nameQuote($c);
				}
			}
		}
		if (!count($cols)) {
			$cols = array('*');
		}
		else if (!in_array('id', $cols)) {
			$cols[] = 'id';
		}
		// build cache key
		$cacheKey = sprintf('%s::%s', $id, implode(':', $cols));
		if (!isset($this->_cache[$cacheKey])) {
			$query = 'SELECT %s FROM #__jmylife_stories WHERE id = %s';
			$this->db->setQuery(sprintf($query, implode(', ', $cols), $id));
			$this->_cache[$cacheKey] = $this->db->loadObject();
			if (0 == intval(@$this->_cache[$cacheKey]->id)) {
				$this->_cache[$cacheKey] = false;
			}
		}
		return $this->_cache[$cacheKey];
	}

	/**
	 * buildQuery
	 * @param   object
	 * @param   int
	 * @param   int
	 * @param   int
	 * @return  string
	 */
	function buildQuery($request, $published=1, $goodonly=0, $countonly=0) {
		$user = JFactory::getUser();
		$fields = array(
			's.*',
			'cat.`name` AS catname',
			'cat.`allow_comments` AS catallowcomments',
			'cat.`allow_votes` AS catallowvotes',
			'cat.`color` AS catcolor',
			'cat.`background` AS catbackground',
			'COUNT( f.`id` ) AS fav',
			'att.url AS attachment_url',
			'att.type AS attachment_type',
			'att.title AS attachment_title',
			'att.description AS attachment_description'
		);
		$orderby = '';
		$ordering = array();
		$group = array('s.`id`');
		$joins = array(
			'LEFT JOIN `#__jmylife_categories` AS cat ON ( cat.`id` = s.`catid` )',
			'LEFT JOIN `#__jmylife_stories_attachments` AS att ON ( att.`sid` = s.`id` )'
		);
		// figure out who has favs
		$whosefavs = $user->id;
		$allowedOthers = array('com_comprofiler', 'com_community');
		if (isset($request['uid']) && in_array(JRequest::getVar('option'), $allowedOthers)) {
			$whosefavs = intval($request['uid']);
		}
		$joins[] = 'LEFT JOIN `#__jmylife_favorites` AS f ON ( f.`uid` = '.$whosefavs.' AND f.`sid` = s.`id` )';
		// only show in published categories
		$wheres = array('cat.`published` = 1');
		// allow -1 to be passed to ignore published status
		if (-1 < $published) {
			$wheres[] = 's.`published` = '.$published;
		}
		// allow -1 to get the bad ones (not in use when added, but just in case)
		if (-1 == $goodonly) {
			$wheres[] = '`reported` = 1';
		}
		else if (0 < $goodonly) {
			$wheres[] = '`reported` = 0';
		}
		// clean view
		@$request['view'] = preg_replace('`[^a-z]*`', '', @$request['view']);
		// handle views
		switch (@$request['view']) {
			case 'favorites':
				$wheres[] = 'f.`id` IS NOT NULL';
				break;
			case 'list':
				$group[] = 'cat.`id`';
				// get allowed cb fields
				$allowedfields = @unserialize($this->cfg->get('cbextra', 'N;'));
				// only grab from a specific category if necessary
				$catid = intval(@$request['catid']);
				if (0 != $catid) {
					$wheres[] = "s.`catid` = $catid";
				}
				// build ORDER BY
				if ('best' == @$request['sort'] || ('vote' == @$request['sort'] && in_array(@$request['sortdir'], array('', 'desc')))) {
					$ordering[] = 's.`votes1` DESC';
					$ordering[] = 's.`votes2` ASC';
					$ordering[] = 's.`time` DESC';
				}
				else if ('worst' == @$request['sort'] || ('vote' == @$request['sort'] && 'asc' == @$request['sortdir'])) {
					$ordering[] = 's.`votes2` DESC';
					$ordering[] = 's.`votes1` ASC';
					$ordering[] = 's.`time` DESC';
				}
				else if ($this->cfg->allow->cb && preg_match('`^cb_`', @$request['sort']) && in_array(@$request['sort'], $allowedfields)) {
					// sanitize sort ;)
					$sort = JRequest::getCmd('sort');
					$field = 'cb.'.$this->db->nameQuote($sort);
					// check sort
					$sortdir = @$request['sortdir'];
					if ('asc' == $sortdir) {
						$sortdir = 'ASC';
					} else {
						$sortdir = 'DESC';
					}
					// add extra field to handle NULL and blanks
					$fields[] = "IF( {$field} IS NULL or {$field}='', 1, 0 ) AS isnull";
					$ordering[] = 'isnull ASC';
					$ordering[] = 'cb.'.$this->db->nameQuote($sort).' '.$sortdir;
					$ordering[] = 's.`time` DESC';
				}
				else {
					if ('asc' == @$request['sortdir']) {
						$ordering[] = 's.`time` ASC';
					} else {
						$ordering[] = 's.`time` DESC';
					}
				}
				// date ranges
				if (isset($request['range1']) && isset($request['range2']) && strlen(@$request['range1'].@$request['range2'])) {
					$range1 = $request['range1'];
					$range2 = $request['range2'];
					// if range1 is 'allbefore' or 'allafter' then use range2 as pivot date and select based on direction
					// quick check to see if we bother
					if (0 === stripos($range1, 'all')) {
						$isBefore = ('allbefore' == $range1);
						$isAfter = ('allafter' == $range1);
						if ($isBefore || $isAfter) {
							$range2 = JFactory::getDate($range2);
							$range2 = intval($range2->toUnix());
							if ($range2) {
								$dir = ($isBefore ? '<' : '>').'=';
								$wheres[] = "s.`time` {$dir} {$range2}";
							}
						}
					}
					// last X days
					else if ('past' == $range1) {
						$range2 = max(1, intval($request['range2']));
						$wheres[] = "FROM_UNIXTIME(s.`time`) > DATE_SUB(NOW(), INTERVAL {$range2} DAY)";
					}
					// year to date
					else if ('ytd' == $range1) {
						$d1 = JFactory::getDate('01-01-'.date('Y'));
						$d2 = JFactory::getDate('01-01-'.(intval(date('Y')) + 1));
						$min = intval($d1->toUnix());
						$max = intval($d2->toUnix());
						$wheres[] = "s.`time` > {$min}";
						$wheres[] = "s.`time` < {$max}";
					}
					// not open ended
					else {
						$d1 = JFactory::getDate($range1);
						$d2 = JFactory::getDate($range2);
						$range1 = intval($d1->toUnix());
						$range2 = intval($d2->toUnix());
						// check if #2 is a date too
						if ($range1) {
							$min = $range1;
							$max = $range1;
							if ($range2) {
								// get everything between the 2
								$min = min($range1, $range2);
								$max = max($range1, $range2);
							}
							$wheres[] = "s.`time` > {$min}";
							$wheres[] = "FROM_UNIXTIME(s.`time`) < DATE_ADD(FROM_UNIXTIME({$max}), INTERVAL 1 DAY)";
						}
					}
				}
				break;
			case 'item':
				// get id
				$id = intval(@$request['id']);
				// decide if an id is needed in WHERE
				if (0 == intval(@$request['random'])) {
					// random is 0, check id
					if (0 == $id) {
						JError::raiseError(404, JText::_("COM_JMYLIFE_ERROR_STORY_NOID"));
					}
					$wheres[] = "s.`id` = $id";
				} else {
					$ordering[] = 'RAND()';
				}
				break;
			case 'user':
				$wheres[] = 's.`user_id` = '.intval(@$request['uid']);
				break;
		}
		// check cb support
		if ($this->cfg->allow->cb) {
			$joins[] = 'LEFT JOIN `#__comprofiler` AS cb'
			         . ' ON ( s.`user_id` = cb.`user_id` ) '
			;
			// avatar
			if ($this->cfg->getInt('show_story_avatar', 0)) {
				$fields[] = 'cb.`avatar`';
			} // end avatar

		} // end cb support
		else if ($this->cfg->allow->jomsocial) {
			$joins[] = 'LEFT JOIN `#__community_users` AS js'
			         . ' ON ( s.`user_id` = js.`userid` ) '
			;
			// avatar
			if ($this->cfg->getInt('show_story_avatar', 0)) {
				$fields[] = 'js.`thumb` AS avatar';
			} // end avatar
		}
		// comment counts
		if ($this->cfg->allow->comments) {
			if ($this->cfg->allow->jcomments) {
				$fields[] = 'COUNT( c.id ) AS numcomments';
				$joins[]  = 'LEFT JOIN `#__jcomments` AS c ON ( c.`object_id` = s.`id` AND c.`object_group` = "com_jmylife" AND c.`published` = 1 )';
			}
			else if ($this->cfg->allow->joomlacomment) {
				$fields[] = 'COUNT( c.id ) AS numcomments';
				$joins[]  = 'LEFT JOIN `#__comment` AS c ON ( c.`contentid` = s.`id` AND c.`component` = "com_jmylife" AND c.`published` = 1 )';
			}
			else if ($this->cfg->allow->jomcomment) {
				$fields[] = 'COUNT( c.id ) AS numcomments';
				$joins[]  = 'LEFT JOIN `#__jomcomment` AS c ON ( c.`contentid` = s.`id` AND c.`option` = "com_jmylife" AND c.`published` = 1 )';
			}
			else if ($this->cfg->allow->disqus) {
				// return 0 for comments
				$fields[] = '0 AS numcomments';
			}
			else {
				$fields[] = 'COUNT( c.id ) AS numcomments';
				$joins[] = 'LEFT JOIN `#__jmylife_comments` AS c'
				           . ' ON ( c.`sid` = s.`id` AND c.`published` = 1 )';
			}
		}
		// build final query
		$query = 'SELECT '
		       . ($countonly ? 'COUNT(*)' : implode(', ', $fields)).' '
		       . 'FROM `#__jmylife_stories` AS s '
		       . implode(' ', $joins)
		       . 'WHERE ( '.implode(' ) AND ( ', $wheres).' ) '
		       . 'GROUP BY '.implode(', ', $group).' '
		       . (count($ordering) ? 'ORDER BY '.implode(',', $ordering) : '')
		       ;
		// all done!
		return $query;
	}

	/**
	 * prepare
	 * @param   object
	 * @param   string
	 * @param   bool
	 * @return  object
	 */
	function prepare($object, $view, $break=true) {
		$appl =& JFactory::getApplication();
		$user =& JFactory::getUser();
		$adminresponse = $this->cfg->get('default_admin_response', '');
		// build links for story & category
		$storyItemid = $this->itemid->story(@$object);
		$categoryItemid = $this->itemid->category(@$object->catid);
		// since Joomla doesn't route properly in admin, make an exception
		// idea modified from http://forum.joomla.org/viewtopic.php?p=2013006#p2013006
		$rawstorylink = 'index.php?option=com_jmylife&view=item&id='.@$object->id.'&Itemid='.$storyItemid;
		$raweditlink  = "{$rawstorylink}&layout=edit";
		$rawcatlink   = 'index.php?option=com_jmylife&view=list&catid='.@$object->catid.'&Itemid='.$categoryItemid;
		if ($appl->isAdmin()) {
			jimport( 'joomla.application.router' );
			require_once (JPATH_ROOT.DS.'includes'.DS.'router.php');
			require_once (JPATH_ROOT.DS.'includes'.DS.'application.php');
			$config    =& JFactory::getConfig();
			$router    =  new JRouterSite(array('mode' => (1 == $config->getValue('config.sef') ? JROUTER_MODE_SEF : '')));
			$storylink =  $router->build($rawstorylink)->toString(array('path', 'query', 'fragment'));
			$editlink  =  $router->build($raweditlink)->toString(array('path', 'query', 'fragment'));
			$catlink   =  $router->build($rawcatlink)->toString(array('path', 'query', 'fragment'));
			// clean administrator/ out of link
			$storylink = str_replace('administrator/', '', $storylink);
			$editlink  = str_replace('administrator/', '', $editlink);
			$catlink   = str_replace('administrator/', '', $catlink);
			$realroot  = str_replace(JURI::root(true), '', JURI::root());
			// TODO: ensure storylink & catlink aren't absolute?
			@$object->storylink     = $this->url->clean($storylink);
			@$object->editlink      = $this->url->clean($editlink);
			@$object->catlink       = $this->url->clean($catlink);
			@$object->fullstorylink = $this->url->clean($realroot.$storylink);
			@$object->fullcatlink   = $this->url->clean($realroot.$catlink);
		} else {
			@$object->storylink     = JRoute::_($rawstorylink, false);
			@$object->editlink      = JRoute::_($raweditlink, false);
			@$object->catlink       = JRoute::_($rawcatlink, false);
			@$object->fullstorylink = JRoute::_($rawstorylink, false, -1);
			@$object->fullcatlink   = JRoute::_($rawcatlink, false, -1);
			// external link frame
			@$object->linkframe     = JRoute::_($rawstorylink.'&format=raw', false);
			@$object->fulllinkframe = JRoute::_($rawstorylink.'&format=raw', false, -1);
		}
		// bugger... the MySQL query needs optimized to include this!
		// check cb support
		if ($this->cfg->allow->cb) {
			// extra fields
			$cbfields = @unserialize($this->cfg->get('cbextra', 'N;'));
			if (0 < count($cbfields) && 0 < @$object->user_id) {
				$query = 'SELECT CONCAT( \'<span class="cbextra">\', GROUP_CONCAT( DISTINCT v.`data` SEPARATOR \'</span><span class="cbextra">\' ), \'</span>\' ) AS cbextra FROM cbextraview AS v LEFT JOIN #__comprofiler AS cb ON ( v.`user_id` = cb.`user_id` ) WHERE v.`user_id` = '.intval(@$object->user_id).' GROUP BY v.`user_id`';
				$this->cfg->db->setQuery($query);
				@$object->cbextra = $this->cfg->db->loadResult();
			} // end cbextra
		} // end cb support
		// save original fulltext
		@$object->rawfulltext = @$object->fulltext.'';
		// only shorten text & create readmore if in list view
		if ('list' == $view) {
			$shorttext = $this->str->shorten(@$object->fulltext, $this->cfg->getInt('story_list_limit', 0));
			@$object->readmore = ($shorttext != @$object->fulltext && $this->cfg->getInt('story_list_limit_max_readmore')) ? $this->html->link(@$object->storylink, JText::_('READMORE'), array('class' => 'jmlReadMore')) : '';
			@$object->fulltext = $shorttext;
		}
		// clean fulltext
		@$object->fulltext = nl2br($this->html->clean(@$object->fulltext));
		// handle admin response
		if (0 < strlen($adminresponse) && 0 == strlen(@$object->admin_fulltext.'')) {
			@$object->admin_fulltext = nl2br($this->html->clean($adminresponse));
		} else {
			@$object->admin_fulltext = nl2br($this->html->clean(@$object->admin_fulltext));
		}
		// clean others
		@$object->user_alias = $this->html->clean(@$object->user_alias);
		@$object->catname = $this->html->clean(@$object->catname);
		@$object->title = $this->html->clean(@$object->title);
		// booleans for votes & comments
		@$object->show_votes = (3 == ($this->cfg->getInt('allow_votes') + (int)(@$object->catallowvotes) + (int)(@$object->allow_votes)));
		@$object->show_comments = (3 == ($this->cfg->getInt('allow_comments') + (int)(@$object->catallowcomments) + (int)(@$object->allow_comments)));
		// break long words
		if ($this->cfg->getInt('break_long_words') && $break) {
			@$object->fulltext   = $this->str->breakLongWords(@$object->fulltext);
			@$object->title      = $this->str->breakLongWords(@$object->title);
			@$object->user_alias = $this->str->breakLongWords(@$object->user_alias);
		}
		// handle bbcode
		@$object->fulltext = $this->html->fromBB(@$object->fulltext);
		// filter naughty words
		@$object->fulltext = $this->str->filterWords(@$object->fulltext);
		// process final avatar image
		@$object->avatar_image = JmylifeCommon::buildAvatarPath(@$object->avatar);
		// process profile link
		@$object->profile_path = JmylifeCommon::buildProfilePath(@$object->user_id);
		// translate dates
		@$object->format_time = date(JText::_('COM_JMYLIFE_STORY_TIMEFORMAT'), @$object->time);
		@$object->format_date = date(JText::_('COM_JMYLIFE_STORY_DATEFORMAT'), @$object->time);
		// item styles
		$style = '';
		if (0 < strlen(@$object->catcolor) || 0 < strlen(@$object->catbackground)) {
			$style .= 'style="';
			if (0 < strlen(@$object->catcolor)) {
				$style .= 'color: #'.@$object->catcolor.';';
			}
			if (0 < strlen(@$object->catbackground)) {
				$style .= 'background: #'.@$object->catbackground.';';
			}
			$style .= '"';
		}
		@$object->style = $style;
		// clean attachment url
		@$object->attachment_url = htmlspecialchars(@$object->attachment_url.'');
		// clean attachment title
		@$object->attachment_title = htmlspecialchars(@$object->attachment_title);
		// clean attachment description
		@$object->attachment_description = nl2br(htmlspecialchars(@$object->attachment_description));
		// get url for attachment image if available
		@$object->attachment_thumb = JmylifeCommon::getStoryAttachmentImageUrl(@$object->id);
		// only allow whitelisted types
		if (!in_array(@$object->attachment_type, array('image', 'link', 'video'))) {
			@$object->attachment_type = 'none';
		}
		@$object->viewerCanEdit = $this->editAllowed($object->id, $user->id);
		return $object;
	}

	/**
	 * prepareList
	 * @param   array
	 * @param   string
	 * @param   bool
	 * @return  array
	 */
	function prepareList($list, $view, $break=true) {
		for ($i=0; $i<count($list); $i++) {
			@$list[$i] = $this->prepare(@$list[$i], $view, $break);
		}
		return $list;
	}

	/**
	 * editAllowed
	 * @param  mixed   story id (or false to use JRequest::getInt('id')
	 * @param  mixed   user id (or false for viewer)
	 * @return bool    true if allowed to edit
	 */
	function editAllowed($id=false, $uid=false) {
		if (false === $id) {
			$id = JRequest::getInt('id', 0);
		} else {
			$id = intval($id);
		}
		if (!$uid) {
			$user = JFactory::getUser();
		} else {
			$user = JUser::getInstance($uid);
		}
		// no guests
		if ($user->guest) {
			return false;
		}
		// not configured
		if (!$this->cfg->getBool('frontend_edit', false)) {
			return false;
		}
		// super admins get a pass in 1.5 ;)
		if ($this->cfg->has->j15()) {
			if (25 == $user->gid) {
				return true;
			}
		}
		// allow if global moderator
		if ($this->cfg->getInt('frontend_edit_group', 20) == $user->gid) {
			return true;
		}
		// if users can't edit own, deny
		if (!$this->cfg->getBool('frontend_edit_userAttachments', false)) {
			return false;
		}
		// load story
		$story = $this->quickLoad($id, 'user_id');
		// story doesn't exist
		if (!$story) {
			return false;
		}
		// by now, any super users will have passed
		if (0 == intval($story->user_id)) {
			return false;
		}
		// check if story belongs to user
		if ($story->user_id == $user->id) {
			return true;
		}
		return false;
	}
}