<?php
/**
 * @package    JMyLife
 * @subpackage Components
 * @version    1.0.16
 * @copyright  2009-2010 Jeff Channell
 * administrator/components/com_jmylife/helpers/url.php
 * @license

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 * @since     1.0.14
 */
// no direct access
defined('_JEXEC') or die('Restricted access');
jimport('joomla.filesystem.file');
if (!class_exists('JmylifeConfiguration')) {
	$configfile = JPATH_ROOT.DS.'components'.DS.'com_jmylife'.DS.'libraries'.DS.'configuration.php';
	if( JFile::exists( $configfile ) ) require_once( $configfile );
}

class JmylifeUrlHelper
{
	private static $_instance;
	var $cfg;

	function __construct() {
		$this->cfg = JmylifeConfiguration::getInstance();
	}

	function getInstance() {
		if (!self::$_instance) {
			self::$_instance = new JmylifeUrlHelper();
		}
		return self::$_instance;
	}

	/**
	 * clean
	 * @param   string
	 * @param   boolean
	 * @return  string
	 */
	function clean($string='', $urlencode=false) {
		$string = preg_replace('`([^\:])\/{2}`', '$1/', $string);
		if ($urlencode) {
			$string = rawurlencode($string);
		}
		return $string;
	}

	/**
	 * validate
	 * @param   string
	 * @return  bool    true if url is valid in structure
	 */
	function validate($url) {
		// this is not exhaustive :/
		return preg_match('`^https?\:/{2}[a-z0-9][a-z0-9\-\.]+?\.[a-z]{2,6}(/\S*)?$`i', $url);
	}

	/**
	 * q
	 * @param   string
	 * @return  string  & or ?
	 *
	 * to be used in building urls
	 */
	function q($url) {
		return (false !== strpos($url, '?') ? '&' : '?');
	}

	/**
	 * isAvailable
	 * @param    string
	 * @return   bool    true if site is accessible, false if not
	 * @from     hobbled together from php docs
	 */
	function isAvailable($url) {
		// may as well make sure url looks kosher before checking
		if (!$this->validate($url)) {
			return false;
		}
		$allowed_responses = array('200', '301');
		if (!in_array('curl', get_loaded_extensions())) {
			$url_parts = @parse_url($url);
			if (empty($url_parts["host"])) {
				return false;
			}
			if (!empty($url_parts["path"])) {
				$documentpath = $url_parts["path"];
			} else {
				$documentpath = "/";
			}
			if (!empty($url_parts["query"])) {
				$documentpath .= "?" . $url_parts["query"];
			}
			$host = $url_parts["host"];
			$port = $url_parts["port"];
			// Now (HTTP-)GET $documentpath at $host";
			if (empty($port)) {
				$port = "80";
			}
			$socket = @fsockopen($host, $port, $errno, $errstr, 30);
			if (!$socket) {
				return false;
			} else {
				fwrite ($socket, "HEAD ".$documentpath." HTTP/1.0\r\nHost: $host\r\n\r\n");
				$http_response = fgets($socket, 22);
				$status = array();
				preg_match('`HTTP\/.* ([0-9]+) .*`', $http_response, $status);
				return in_array($status[1], $allowed_responses);
			}
			return false;
		}
		$ch = @curl_init($url);
		@curl_setopt($ch, CURLOPT_HEADER, TRUE);
		@curl_setopt($ch, CURLOPT_NOBODY, TRUE);
		if (!(@ini_get('open_basedir') || @ini_get('safe_mode'))) {
			@curl_setopt($ch, CURLOPT_FOLLOWLOCATION, FALSE);
		}
		@curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
		$status = array();
		$http_response = @curl_exec($ch);
		preg_match('/HTTP\/.* ([0-9]+) .*/', $http_response, $status);
		return in_array( $status[1], $allowed_responses );
	}

}