<?php
/**
 * @package    JMyLife
 * @subpackage Components
 * @version    1.0.16
 * @copyright  2009-2010 Jeff Channell
 * administrator/components/com_jmylife/models/comments.php
 * @license

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 */
// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );
// loaded since 1.0.5
require_once( JPATH_COMPONENT_SITE.DS.'libraries'.DS.'common.php' );
/**
 * Comments Model
 *
 * @package    JMyLife
 * @subpackage Components
 */
class JmylifeModelComments extends JModel
{
	/**
	 * data
	 * @var stdClass
	 */
	var $_data = null;

	/**
	 * Items total - for pagination
	 * @var integer
	 */
	var $_total = null;

	/**
	 * Pagination object
	 * @var object
	 */
	var $_pagination = null;

	/**
	 * category list
	 * @var array
	 */
	var $_categories = null;

	/**
	 * contructor
	 */
	function __construct()
	{
		parent::__construct();

		// J!1.6 doesn't use mainframe
		$appl =& JFactory::getApplication();
		global $option, $context;

		// Get pagination request variables
		$limit = $appl->getUserStateFromRequest( 'global.list.limit', 'limit', $appl->getCfg( 'list_limit' ), 'int' );
		$limitstart = JRequest::getVar( 'limitstart', 0, '', 'int' );
		// In case limit has been changed, adjust it
		$limitstart = ( $limit != 0 ? ( floor( $limitstart / $limit ) * $limit ) : 0 );
		// set in state
		$this->setState( 'limit', $limit );
		$this->setState( 'limitstart', $limitstart );

		// a little cleanup
		$filter_view       = $appl->getUserStateFromRequest( $option.$view.'filter_view', 'filter_view', 'comments', 'cmd' );
		if( 'comments' != $filter_view )
		{
			$appl->setUserState( $option.$view.'filter_view', 'comments' );
			$appl->setUserState( $option.$view.'filter_order', 'c.`time`' );
			$appl->setUserState( $option.$view.'filter_order_Dir', 'DESC' );
		}
		// get filter order
		$filter_order      = $appl->getUserStateFromRequest( $option.$view.'filter_order', 'filter_order', 'c.`time`', 'cmd' );
		$filter_order_Dir  = $appl->getUserStateFromRequest( $option.$view.'filter_order_Dir', 'filter_order_Dir', 'DESC', 'word' );
		// filter by category
		$filter_categoryid = $appl->getUserStateFromRequest( $option.$view.'filter_categoryid', 'filter_categoryid', -1, 'int' );
		// filter by search
		$search = $appl->getUserStateFromRequest( $context.'search', 'search', '', 'string' );
		$search = JString::strtolower( $search );
		// set in state
		$this->setState( 'order', $filter_order );
		$this->setState( 'order_Dir', $filter_order_Dir );
		$this->setState( 'categoryid', $filter_categoryid );
		$this->setState( 'search', $search );
  }

  function _buildContentOrderBy()
  {
  	$orderby = ' ORDER BY '.$this->getState('order').' '.$this->getState('order_Dir').' ';
  	return $orderby;
	}

  function _buildContentWhere()
  {
  	$database   =& JFactory::getDBO();
  	$search     = $this->getState( 'search' );
  	$where = array();
  	if( strlen( $search ) > 0 )
  	{
  		$where[] = ' ( LOWER( c.`message` ) LIKE '.$database->Quote( '%'.$database->getEscaped( $search, true ).'%', false ) .
				' OR c.id = ' . (int) $search . ')';
		}
		if( count( $where ) == 0 )
			$where[] = '1';
  	return $where;
	}


	/**
	 * builds item query
	 * @return string
	 *
	 */
	function _buildItemQuery()
	{
		$_JMLConfig = JmylifeConfiguration::getInstance();
		// initialize
		$orderby = '';
		$join = 'LEFT JOIN `#__jmylife_comments_warnings` AS w'
		      . ' ON ( c.`id` = w.`cid` ) '
		      // since 1.0.5
		      . 'LEFT JOIN `#__jmylife_stories` AS s'
		      . ' ON ( s.`id` = c.`sid` ) '
		      ;
		$group = ' GROUP BY w.`cid` ';
		$fields = array( 'c.*', 'COUNT( w.`ip` ) AS warninglevel', 'CONCAT(LEFT( s.`fulltext`, 47 ), "...") AS storytext' );
		$user =& JFactory::getUser();
		$cid = JRequest::getInt( 'cid' );
		$where = array( 'c.`id`='.$cid );
		if( $_JMLConfig->getInt( 'show_comment_avatar' ) )
		{
			// check for community builder support
			if( $_JMLConfig->allow->cb )
			{
				// add extra fields
				$fields[] = 'cb.`avatar` AS avatar';
				// change query to join
				$join.= 'LEFT JOIN `#__comprofiler` AS cb'
					. ' ON ( c.`author_id` = cb.`user_id` ) '
					;
				$group .= ', c.`id` ';
			}
			else if( $_JMLConfig->allow->jomsocial )
			{
				// add extra fields
				$fields[] = 'js.`thumb` AS avatar';
				// change query to join
				$join.= 'LEFT JOIN `#__community_users` AS js'
					. ' ON ( c.`author_id` = js.`userid` ) '
					;
				$group .= ', c.`id` ';
			}
		}
		// build query
		$query = 'SELECT '
		       . implode( ', ', $fields ).' '
		       . 'FROM `#__jmylife_comments` AS c '
		       . $join
		       . ' WHERE ( '.implode( ' ) AND ( ', $where ).' ) '
		       . ' '.$group.' '
		       . ' '.$orderby.' '
		       ;
		return $query;
	}

	/**
	 * builds query
	 * @return string
	 *
	 */
	function _buildQuery()
	{
		$_JMLConfig = JmylifeConfiguration::getInstance();
		// initialize
		$join = 'LEFT JOIN `#__jmylife_comments_warnings` AS w'
		      . ' ON ( c.`id` = w.`cid` ) '
		      // since 1.0.5
		      . 'LEFT JOIN `#__jmylife_stories` AS s'
		      . ' ON ( s.`id` = c.`sid` ) '
		      ;
		$group = ' GROUP BY c.`id` ';
		$fields = array( 'c.*', 'COUNT( w.`ip` ) AS warninglevel', 'CONCAT(LEFT( s.`fulltext`, 47 ), "...") AS storytext' );
		$orderby = $this->_buildContentOrderBy();
		$where = $this->_buildContentWhere();
		// check for community builder support
		if( $_JMLConfig->getInt( 'show_comment_avatar' ) )
		{
			if( $_JMLConfig->allow->cb )
			{
				// add extra fields
				$fields[] = 'cb.`avatar`';
				// change query to join
				$join.= 'LEFT JOIN `#__comprofiler` AS cb'
					. ' ON ( c.`author_id` = cb.`user_id` ) '
					;
				$group.= ', c.`id` ';
			}
			else if( $_JMLConfig->allow->jomsocial )
			{
				// add extra fields
				$fields[] = 'js.`thumb` AS avatar';
				// change query to join
				$join.= 'LEFT JOIN `#__community_users` AS js'
					. ' ON ( c.`author_id` = js.`userid` ) '
					;
				$group .= ', c.`id` ';
			}
		}
		// build query
		$query = 'SELECT '
		       . implode( ', ', $fields ).' '
		       . 'FROM `#__jmylife_comments` AS c '
		       . $join
		       . ' WHERE ('.implode( ') AND (', $where ).') '
		       . ' '.$group.' '
		       . ' '.$orderby.' '
		       ;
		return $query;
	}

	function getSearch()
	{
		return $this->getState( 'search' );
	}

	/**
	 * Gets the list data
	 * @return stdClass object of listings
	 *
	 * @todo use Joomla Tables instead of simple queries
	 */
	function getData()
	{
		// if data hasn't already been obtained, load it
		if( empty( $this->_data ) )
		{
			$query = $this->_buildQuery();
			$this->_data = $this->_getList( $query, $this->getState( 'limitstart' ), $this->getState( 'limit' ) );
		}
		return $this->_data;
	}

	/**
	 * Gets the item data
	 * @return stdClass object of listings
	 *
	 * @todo use Joomla Tables instead of simple queries
	 */
	function getItem()
	{
		// if data hasn't already been obtained, load it
		if( empty( $this->_data ) )
		{
			$query = $this->_buildItemQuery();
			$this->_data = $this->_getList( $query, $this->getState( 'limitstart' ), $this->getState( 'limit' ) );
		}
		return $this->_data;
	}

	/**
	 * Gets the total amount of listings
	 * @return int
	 */
	function getTotal()
	{
		// Load the content if it doesn't already exist
		if( empty( $this->_total ) )
		{
			$query = $this->_buildQuery();
			$this->_total = $this->_getListCount( $query );
		}
		return $this->_total;
	}

	/**
	 * Gets the listings pagination
	 * @return JPagination object
	 */
	function getPagination()
	{
		// Load the content if it doesn't already exist
		if( empty( $this->_pagination ) )
		{
			jimport( 'joomla.html.pagination' );
			$this->_pagination = new JPagination( $this->getTotal(), $this->getState( 'limitstart' ), $this->getState( 'limit' ) );
		}
		return $this->_pagination;
	}
}