<?php
/**
 * @package    JMyLife
 * @subpackage Components
 * @version    1.0.16
 * @copyright  2009-2010 Jeff Channell
 * administrator/components/com_jmylife/models/configuration.php
 * @license

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 */
// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );
jimport('joomla.filesystem.file');
jimport('joomla.filesystem.folder');

/**
 * Configuration Model
 *
 * @package    JMyLife
 * @subpackage Components
 */
class JmylifeModelConfiguration extends JModel
{
	/**
	 * config list
	 * @var array
	 */
	var $_config = null;

	var $_comment_engines = null;
	var $_captcha_engines = null;
	var $_recaptcha_themes = null;
	var $_graphical_voting_size = null;
	var $_social_engines = null;
	var $_cbextra = null;
	var $_images = null;
	var $_sharethis = null;

	/**
	 * contructor
	 */
	function __construct()
	{
		parent::__construct();
  }
	/**
	 * builds query
	 * @return string
	 *
	 */
	function _buildQuery()
	{
		// build query
		// ordering & sorting since 1.0.4
		// language join in 1.0.11
		// NOTE: each language needs an extry in config!
		return 'SELECT c.`key`, c.`group`, c.`subgroup`, c.`ordering`, c.`type`, c.`label`, c.`description`, IF( ln.`value` IS NOT NULL, ln.`value`, c.`value` ) AS value FROM `#__jmylife_configuration` AS c LEFT JOIN `#__jmylife_language` AS ln ON ( CONVERT( c.`key` USING utf8 ) = CONVERT( ln.`key` USING utf8 ) ) GROUP BY c.`key` ORDER BY c.`group` ASC, c.`subgroup` ASC, c.`ordering` ASC, c.`key` ASC ';
	}

	/**
	 * builds group query
	 * @return string
	 * @since 1.0.5
	 *
	 */
	function _buildGroupQuery()
	{
		// build query
		// ordering & sorting since 1.0.4
		$query = 'SELECT '
		       . '`group` '
		       . 'FROM `#__jmylife_configuration` '
		       . 'GROUP BY `group` '
		       ;
		return $query;
	}

	/**
	 * Gets the list data
	 * @return stdClass object of listings
	 *
	 * @todo use Joomla Tables instead of simple queries
	 */
	function getConfig()
	{
		// if data hasn't already been obtained, load it
		if( empty( $this->_config ) )
		{
			$query = $this->_buildQuery();
			$this->_config = $this->_getList( $query, 0, 99999 );
		}
		return $this->_config;
	}

	/**
	 * Gets groups
	 * @return stdClass object of listings
	 * @since 1.0.5
	 *
	 */
	function getGroups()
	{
		// if data hasn't already been obtained, load it
		if( empty( $this->_groups ) )
		{
			$query = $this->_buildGroupQuery();
			$this->_groups = $this->_getList( $query, 0, 99999 );
		}
		return $this->_groups;
	}
	
	/**
	 * Gets comment engines
	 * @return string  html string for special config key 'comment_engine'
	 * @since 1.0.5
	 *
	 */
	function getCommentEngines()
	{
		$_JMLConfig = JmylifeConfiguration::getInstance();
		// if data hasn't already been obtained, load it
		if( empty( $this->_comment_engines ) )
		{
			$engines = array();
			// always add JMyLife core comment engine
			$engines[] = JHTML::_('select.option', '0', JText::_('COM_JMYLIFE_CONFIGURATION_COMMENTOPTION_CORE') );
			// Disqus added in 1.0.11
			// there's nothing really to check here as it's all externally driven
			$engines[] = JHTML::_('select.option', '3', JText::_('COM_JMYLIFE_CONFIGURATION_COMMENTOPTION_DISQUS') );
			// detect jcomments
			if( $_JMLConfig->has->jcomments )
			{
				$engines[] = JHTML::_('select.option', '1', JText::_('COM_JMYLIFE_CONFIGURATION_COMMENTOPTION_JCOMMENTS') );
			}
			// detect joomlacomment
			if( $_JMLConfig->has->joomlacomment )
			{
				$engines[] = JHTML::_('select.option', '2', JText::_('COM_JMYLIFE_CONFIGURATION_COMMENTOPTION_COMPOJOOMCOMMENT') );
			}
			// detect jomcomment, since 1.0.12
			if( $_JMLConfig->has->jomcomment )
			{
				$engines[] = JHTML::_('select.option', '4', JText::_('COM_JMYLIFE_CONFIGURATION_COMMENTOPTION_JOMCOMMENT') );
			}
			// TODO: add more engines ;)
			// all engines detected
			$this->_comment_engines = JHTML::_( 'select.genericlist', $engines, 'comment_engine', '', 'value', 'text', $_JMLConfig->getInt('comment_engine',0), 'comment_engine' );
		}
		return $this->_comment_engines;
	}

	/**
	 * Gets captcha engines
	 * @return string  html string for special config key 'captcha_engines'
	 * @since 1.0.11
	 */
	function getCaptchaEngines()
	{
		$_JMLConfig = JmylifeConfiguration::getInstance();
		if( empty( $this->_captcha_engines ) )
		{
			$choices = array(
				JHTML::_('select.option', '0', JText::_('COM_JMYLIFE_CONFIGURATION_CAPTCHAOPTION_NONE') ),
				JHTML::_('select.option', '1', JText::_('COM_JMYLIFE_CONFIGURATION_CAPTCHAOPTION_RECAPTCHA') ),
				JHTML::_('select.option', '3', JText::_('COM_JMYLIFE_CONFIGURATION_CAPTCHAOPTION_CAPTCHASNET') )
			);
			// check for BigoCaptcha
			if( JFile::exists( JPATH_ROOT.DS.'plugins'.DS.'system'.DS.'bigocaptcha.php' ) )
			{
				$choices[] = JHTML::_('select.option', 2, JText::_('COM_JMYLIFE_CONFIGURATION_CAPTCHAOPTION_BIGO') );
			}
			$this->_captcha_engines = JHTML::_( 'select.genericlist', $choices, 'captcha_engine', '', 'value', 'text', $_JMLConfig->getInt( 'captcha_engine', 0 ), 'captcha_engine' );
		}
		return $this->_captcha_engines;
	}

	/**
	 * Gets recaptcha themes
	 * @return string  html string for special config key 'recaptcha_theme'
	 * @since 1.0.9
	 */
	function getRecaptchaThemes()
	{
		$_JMLConfig = JmylifeConfiguration::getInstance();
		if( empty( $this->_recaptcha_themes ) )
		{
			$themes = array( 'red', 'white', 'blackglass', 'clean' );
			$choices = array();
			foreach( $themes as $theme )
			{
				$choices[] = JHTML::_('select.option', $theme, JText::_('COM_JMYLIFE_CONFIGURATION_RECAPTCHATHEME_'.strtoupper($theme)) );
			}
			$this->_recaptcha_themes = JHTML::_( 'select.genericlist', $choices, 'recaptcha_theme', '', 'value', 'text', $_JMLConfig->get( 'recaptcha_theme', 'red' ), 'recaptcha_theme' );
		}
		return $this->_recaptcha_themes;
	}

	/**
	 * Gets all css files in custom css folder
	 * @return  string  html string for a select box for each file
	 * @since 1.0.9
	 */
	function getCustomCssFiles()
	{
		$_JMLConfig = JmylifeConfiguration::getInstance();
		if( empty( $this->_css_files ) )
		{
			// check folder, then open, read all css, and spit out a string for html
			$folder = JPATH_SITE.DS.'components'.DS.'com_jmylife'.DS.'assets'.DS.'css'.DS.'custom'.DS;
			if( !JFolder::exists( $folder ) )
			{
				if( !JFolder::create( $folder ) ) return '<span style="color:red;font-weight:bold;">'.JText::_('COM_JMYLIFE_ERROR_CONFIGURATION_CUSTOMCSS_FOLDERDOESNOTEXIST').'</span>';
			}
			$files = JFolder::files( $folder, '\.css$' );
			$choices = array( JHTML::_('select.option', '0', JText::_('COM_JMYLIFE_CONFIGURATION_CUSTOMCSS_NOCUSTOMOPTION') ) );
			foreach( $files as $file )
			{
				$choices[] = JHTML::_('select.option', $file, $file, 'value', 'text', (bool)!( JFile::write( $folder.$file, JFile::read( $folder.$file ) ) ) );
			}
			$this->_css_files = JHTML::_( 'select.genericlist', $choices, 'css_custom_styles', '', 'value', 'text', $_JMLConfig->get( 'css_custom_styles', '' ), 'css_custom_styles' );
		}
		return $this->_css_files;
	}

	/**
	 * Gets the contents of selected custom css
	 * @since 1.0.9
	 */
	function getCustomCss()
	{
		$_JMLConfig = JmylifeConfiguration::getInstance();
		$file = $_JMLConfig->get( 'css_custom_styles', '' );
		if( !preg_match( '`\.css$`', $file ) ) return '';
		$path = realpath( JPATH_SITE.DS.'components'.DS.'com_jmylife'.DS.'assets'.DS.'css'.DS.'custom'.DS.$file );
		if( JFile::exists( $path ) ) return JFile::read( $path );
		else return '';
	}

	/**
	 * gets the contents if /images/ for header logo
	 * @since 1.0.13
	 */
	function getStoryLinkImages()
	{
		$_JMLConfig = JmylifeConfiguration::getInstance();
		if( empty( $this->_images ) )
		{
			// get path to site images folder
			$folder = JPATH_SITE.DS.'images'.DS.'stories'.DS.'jmylife'.DS;
			if( !JFolder::exists( $folder ) )
			{
				if( !JFolder::create( $folder ) ) return '<span style="color:red;font-weight:bold;">'.JText::_('COM_JMYLIFE_ERROR_CONFIGURATION_ATTACHMENTS_IMAGEFOLDERDOESNOTEXIST').'</span>';
			}
			$files = JFolder::files( $folder/*, '\.[jpe?g|gif|png]$'*/ );
			$choices = array( JHTML::_('select.option', '', ' - default - ' ) );
			foreach( $files as $file )
			{
				$choices[] = JHTML::_('select.option', $file, $file);
			}
			$this->_images = JHTML::_( 'select.genericlist', $choices, 'storylink_header_logo', '', 'value', 'text', $_JMLConfig->get( 'storylink_header_logo', '' ), 'css_custom_styles' );
		}
		return $this->_images;

	}

	/**
	 * Builds a select element for graphical vote sizes
	 * @since 1.0.10
	 */
	function getGraphicalVotingSize()
	{
		$_JMLConfig = JmylifeConfiguration::getInstance();
		if( empty( $this->_graphical_voting_size ) )
		{
			$choices = array(
				JHTML::_('select.option', '48', JText::_('COM_JMYLIFE_CONFIGURATION_GRAPHICALVOTINGOPTION_LARGE') ),
				JHTML::_('select.option', '36', JText::_('COM_JMYLIFE_CONFIGURATION_GRAPHICALVOTINGOPTION_MEDIUM') ),
				JHTML::_('select.option', '24', JText::_('COM_JMYLIFE_CONFIGURATION_GRAPHICALVOTINGOPTION_SMALL') ),
			);
			$this->_graphical_voting_size = JHTML::_( 'select.genericlist', $choices, 'graphical_voting_size', '', 'value', 'text', $_JMLConfig->get( 'graphical_voting_size', '48' ), 'graphical_voting_size' );
		}
		return $this->_graphical_voting_size;
	}
	
	/**
	 * Builds multiselect for social engine fields
	 * @since 1.0.12
	 */
	function getSocialEngineFields()
	{
		$_JMLConfig = JmylifeConfiguration::getInstance();
		if( empty( $this->_social_engines ) )
		{
			$choices = array( JHTML::_('select.option', '0', JText::_('COM_JMYLIFE_CONFIGURATION_SOCIALENGINEOPTION_NONE') ) );
			if( $_JMLConfig->has->cb )
			{
				$choices[] = JHTML::_('select.option', '1', JText::_('COM_JMYLIFE_CONFIGURATION_SOCIALENGINEOPTION_CB') );
			}
			if( $_JMLConfig->has->jomsocial )
			{
				$choices[] = JHTML::_('select.option', '2', JText::_('COM_JMYLIFE_CONFIGURATION_SOCIALENGINEOPTION_JS') );
			}
			$this->_social_engines = JHTML::_( 'select.genericlist', $choices, 'social_engine', '', 'value', 'text', $_JMLConfig->get( 'social_engine', 0 ), 'social_engine' );
		}
		return $this->_social_engines;
	}

	/**
	 * Builds multiselect for Community Builder fields
	 * @since 1.0.11
	 */
	function getCommunityBuilderFields()
	{
		$_JMLConfig = JmylifeConfiguration::getInstance();
		if( empty( $this->_cbextra ) )
		{
			$choices = array();
			// no 500 error in debug mode - 1.0.12
			if( $_JMLConfig->has->cb  )
			{
				$_JMLConfig->db->setQuery( 'SELECT `name` FROM `#__comprofiler_fields` WHERE `table` LIKE "%_comprofiler" AND `published` = 1 AND `name` LIKE "cb_%"' );
				if( $fields = $_JMLConfig->db->loadResultArray() )
				{
					if( count( $fields ) )
					{
						foreach( $fields as $field )
						{
							$choices[] = JHTML::_('select.option', $field, $field );
						}
					}
				}
			}
			$this->_cbextra = JHTML::_( 'select.genericlist', $choices, 'cbextra[]', 'class="inputbox" multiple="multiple" size="8"', 'value', 'text', unserialize( $_JMLConfig->get( 'cbextra', '' ) ), 'cbextra' );
		}
		return $this->_cbextra;
	}

	/**
	 * Builds multiselect for ShareThis layout types
	 */
	function getShareThisLayoutTypes() {
		$cfg = JmylifeConfiguration::getInstance();
		if (empty($this->_sharethis)) {
			$choices = array(
				// single span, no suffix
				// <span class="st_sharethis" displayText="ShareThis"></span>
				JHTML::_('select.option', 'classic', JText::_('COM_JMYLIFE_CONFIGURATION_SHARETHISOPTION_CLASSIC')),
				// multispan, no suffix
				// <span class="st_email"></span><span class="st_facebook"></span><span class="st_twitter"></span><span class="st_sharethis" displayText="ShareThis"></span>
				JHTML::_('select.option', 'varmini', JText::_('COM_JMYLIFE_CONFIGURATION_SHARETHISOPTION_VARIETYMINI')),
				// multispan, _large suffix
				// <span class="st_twitter_large" displayText="Tweet"></span><span class="st_facebook_large" displayText="Facebook"></span><span class="st_ybuzz_large" displayText="Yahoo! Buzz"></span><span class="st_gbuzz_large" displayText="Google Buzz"></span><span class="st_email_large" displayText="Email"></span><span class="st_sharethis_large" displayText="ShareThis"></span>
				JHTML::_('select.option', 'varsuper', JText::_('COM_JMYLIFE_CONFIGURATION_SHARETHISOPTION_VARIETYSUPER')),
				// multispan, _button suffix
				// <span class="st_facebook_button" displayText="Facebook"></span><span class="st_twitter_button" displayText="Tweet"></span><span class="st_email_button" displayText="Email"></span>
				JHTML::_('select.option', 'varstandard', JText::_('COM_JMYLIFE_CONFIGURATION_SHARETHISOPTION_VARIETYSTANDARD')),
				// multispan, _hcount suffix
				// <span class="st_twitter_hcount" displayText="Tweet"></span><span class="st_facebook_hcount" displayText="Share"></span><span class="st_email_hcount" displayText="Email"></span><span class="st_sharethis_hcount" displayText="Share"></span>
				JHTML::_('select.option', 'horizontal', JText::_('COM_JMYLIFE_CONFIGURATION_SHARETHISOPTION_HORIZONTAL'))/*,
				// multispan, _vcount suffix
				// <span class="st_twitter_vcount" displayText="Tweet"></span><span class="st_facebook_vcount" displayText="Share"></span><span class="st_email_vcount" displayText="Email"></span><span class="st_sharethis_vcount" displayText="Share"></span>
				JHTML::_('select.option', 'vertical', 'Vertical Skyscrapers')*/
			);
			$this->_sharethis = JHTML::_('select.genericlist', $choices, 'sharethislayout', 'class="inputbox"', 'value', 'text', $cfg->get('sharethislayout', 'classic'), 'sharethislayout');
		}
		return $this->_sharethis;
	}

}