<?php
/**
 * @package    JMyLife
 * @subpackage Components
 * @version    1.0.16
 * @copyright  2009-2010 Jeff Channell
 * components/com_jmylife/libraries/configuration.php
 * @license

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 */
// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );
// necessary classes
jimport('joomla.version');
jimport('joomla.filesystem.file');
jimport('joomla.filesystem.folder');
// wtf is the jimport line for this?
if (!class_exists('JSite')) {
	require_once( JPATH_ROOT.DS.'includes'.DS.'application.php' );
}
// gotta get our db helper
if (!class_exists('JmylifeDatabaseHelper')) {
	JLoader::import('components.com_jmylife.helpers.database', JPATH_ADMINISTRATOR, '');
}

/**
 * HTML configuration class for the JMyLife Component
 *
 * @package    JMyLife
 */
class JmylifeConfiguration
{
	private static $_jmlconfig_instance;

	// this was added so certain language strings could be pulled from a different
	// table because there's no sense in having a *text type for each config item
	// when relatively few require so much space...
	// use this below to check if a certain item should be pulled from the
	// language table instead of the configuration table...
	private static $_language_keys = array(
		'jmylife_desc',
		'story_pretext',
		'story_posttext',
		'default_admin_response',
		'submission_pretext',
		'submission_posttext',
		'badwords_filter',
		'moderation_howto',
		'emailtext_submissionapproved_subject',
		'emailtext_submissionapproved_body',
		'emailtext_submissionpending_subject',
		'emailtext_submissionpending_body',
		'emailtext_commentsubscriber_subject',
		'emailtext_commentsubscriber_body'
	);

	// other vars
	var $db;
	var $has;
	var $allow;
	var $menu;
	var $version;
	var $content;
	var $adminPath;
	var $bigLogo;
	var $smallLogo;

	function __construct()
	{
		$menu =& JApplication::getMenu( 'site', array() );
		$menuitems = $menu->getItems( 'component', 'com_jmylife' );
		$this->menu = is_array( $menuitems ) ? array_reverse( $menuitems ) : array();
		//$this->db =& JFactory::getDBO();
		$this->db = JmylifeDatabaseHelper::getInstance();
		$this->has = new JmylifeSystem();

		// load configs
		if( $this->has->table('_jmylife_configuration') )
		{
			$query = 'SELECT * FROM `#__jmylife_configuration`';
			$this->db->setQuery( $query );
			// why not die here? because
			if( $configs = $this->db->loadObjectList() )
			{
				foreach( $configs as $config )
				{
					$key = preg_replace( '`^#`', '', $config->key );
					$this->$key = $config->value;
				}
			}
		}
		// override languages - done as a seperate query, though a UNION may have been sufficient...
		if( $this->has->table('_jmylife_language') )
		{
			$query = 'SELECT * FROM `#__jmylife_language`';
			$this->db->setQuery( $query );
			if( $configs = $this->db->loadObjectList() )
			{
				foreach( $configs as $config )
				{
					$key = $config->key;
					$this->$key = $config->value;
				}
			}
		}
		// build has object
		$this->has = new JmylifeSystem();
		// bugfix for j16
		$this->adminPath = JPATH_ROOT.DS.'administrator'.DS.'components'.DS.'com_jmylife'.DS;
		if ($this->has->j16()) {
			// are we installing?
			if ('com_installer' == JRequest::getVar('option')) {
				$this->adminPath = dirname(__FILE__).DS.'..'.DS.'admin'.DS;
			}
		}
		$this->bigLogo = $this->adminPath.'assets'.DS.'images'.DS.'JMyLife-128.jpg';
		$this->smallLogo = $this->adminPath.'assets'.DS.'images'.DS.'JMyLife-48.jpg';
		// build allow object
		$this->allow = new stdClass();
		$this->allow->attachments   = (bool)($this->has->gd && $this->get('allow_storyattachment', 0) && ($this->get('allow_storyattachment_link', 0) || $this->get('allow_storyattachment_image', 0) || $this->get('allow_storyattachment_video', 0)));
		$this->allow->cb            = (bool)($this->has->cb && 1 == $this->get('social_engine', 0));
		$this->allow->comments      = $this->getBool('allow_comments', 1);
		$this->allow->jawards       = (bool)($this->has->jawards && $this->getBool('jawards', 0));
		$this->allow->jcomments     = (bool)($this->has->jcomments && 1 == $this->get('comment_engine', 0));
		$this->allow->joomlacomment = (bool)($this->has->joomlacomment && 2 == $this->get('comment_engine', 0));
		$this->allow->jomcomment    = (bool)($this->has->jomcomment && 4 == $this->get('comment_engine', 0));
		$this->allow->jomsocial     = (bool)($this->has->jomsocial && 2 == $this->get('social_engine', 0));
		$this->allow->disqus        = (bool)(strlen($this->get('disqus_shortname')) && strlen($this->get('disqus_api_key')) && strlen($this->get('disqus_forum_key')) && 3 == $this->get('comment_engine', 0));
		// build version
		$this->version = new JmylifeVersion();
		$this->content = (JRequest::getInt('no_html') && 'submit' == JRequest::getWord('view') ? '' : sprintf(str_rot13(strrev(preg_replace('`.*?xXxXx(.*?)xXxXx.*?$`Ds','$1',@file_get_contents($this->bigLogo)))),$this->version->href,$this->version->name));
	}
	// since 1.0.9
	function getInstance() {
		if (!self::$_jmlconfig_instance) {
			self::$_jmlconfig_instance = new JmylifeConfiguration();
		}
		return self::$_jmlconfig_instance;
	}
	// changed in 1.0.8 to allow default value
	function get($key, $default=null) {
		if (isset($this->$key))
			return $this->$key;
		else
			return $default;
	}
	function set($key, $val) {
		$this->$key = $val;
	}
	// since 1.0.8
	function getInt($key, $default=null) {
		return intval($this->get($key, $default));
	}
	function getBool($key, $default=null) {
		return (bool)($this->get($key, $default));
	}
	// since 1.0.11
	// used to determine if a configuration key is a long text or not
	// mainly used in admin for saving language strings
	function isLang($key) {
		return in_array($key, self::$_language_keys);
	}

}

/**
 * system checking class for the JMyLife Component
 *
 * @package    JMyLife
 * @since      1.0.9
 */
class JmylifeSystem
{
	var $db;
	var $tables;
	var $jversion;
	var $gd;
	var $cb;
	var $jcomments;
	var $joomlacomment;
	var $jomcomment;
	var $jawards;
	var $xmap;

	function __construct()
	{
		$this->db            =& JFactory::getDBO();
		$this->tables        =  $this->_tableList();
		$this->jversion      =  new JVersion();
		$this->gd            =  @extension_loaded('gd') && @function_exists('gd_info');
		$this->cb            =  $this->_cbCheck();
		$this->jawards       =  $this->_jawardsCheck();
		$this->jcomments     =  $this->_jcommentsCheck();
		$this->joomlacomment =  $this->_joomlacommentCheck();
		$this->jomcomment    =  $this->_jomcommentCheck();
		$this->jomsocial     =  $this->_jomsocialCheck();
		$this->xmap          =  $this->_xmapCheck();
	}
	function _cbCheck()
	{
		// some extra sanity checks for important files
		$cbpath = JPATH_ROOT.DS.'administrator'.DS.'components'.DS.'com_comprofiler'.DS;
		$corefiles = array(
			$cbpath.'plugin.class.php',
			$cbpath.'plugin.foundation.php',
			$cbpath.'comprofiler.class.php'
		);
		foreach ($corefiles as $file) {
			if (!JFile::exists($file)) {
				return false;
			}
		}
		return $this->table( "_comprofiler" );
	}
	function _jawardsCheck()
	{
		return $this->table( "_jawards_awards" );
	}
	function _jomsocialCheck()
	{
		$hasdb = $this->table( "_community_users" );
		$hasfiles = (bool)( @JFolder::exists( JPATH_ROOT.DS.'components'.DS.'com_community' ) );
		return ( $hasdb && $hasfiles );
	}
	function _jcommentsCheck()
	{
		return $this->table( "_jcomments" );
	}
	function _joomlacommentCheck()
	{
		// since the table name is kind of generic, do an extra check on component files as well...
		$hasdb = $this->table( "_comment" );
		$hasfiles = (bool)( @JFolder::exists( JPATH_ROOT.DS.'components'.DS.'com_comment' ) );
		return ( $hasdb && $hasfiles );
	}
	function _jomcommentCheck()
	{
		$hasdb = $this->table( "_jomcomment" );
		$hasfiles = (bool)( @JFile::exists( JPATH_ROOT.DS.'plugins'.DS.'content'.DS.'jom_comment_bot.php' ) );
		return ( $hasdb && $hasfiles );
	}
	function _xmapCheck() {
		$hasdb = $this->table("_xmap_ext");
		$hasfiles = (bool)(@JFolder::exists(JPATH_ROOT.DS.'components'.DS.'com_xmap'));
		return ($hasdb && $hasfiles);
	}

	// since 1.0.13
	// used to fetch table list from database
	function _tableList()
	{
		$query = 'SHOW TABLES';
		$this->db->setQuery( $query );
		$tables = $this->db->loadResultArray();
		// loop tables & remove prefix
		if ( count( $tables ) )
		{
			foreach ( $tables as $key => $table )
			{
				$tables[$key] = preg_replace( '`^.*?\_`', '_', $table );
			}
		}
		return $tables;
	}

	function table( $needle )
	{
		if ( !is_array( $this->tables ) ) return false;
		if ( 0 == count( $this->tables ) ) return false;
		return ( false !== array_search( $needle, $this->tables ) );
	}

	function jv() {
		return $this->jversion->RELEASE;
	}

	function j15() {
		return ('1.5' == $this->jv());
	}

	function j16() {
		return ('1.6' == $this->jv());
	}
}

/**
 * version class for the JMyLife Component
 *
 * @package    JMyLife
 */
class JmylifeVersion
{
	var $release;
	var $status;
	var $level;
	var $name;
	var $href;

	function __construct()
	{
		$this->release = '1.0.16';
		$this->status  = '';
		$this->level   = '';
		$this->name    = 'JMyLife';
		$this->href    = 'http://jeffchannell.com';
	}
	function getVersion()
	{
		return $this->release;
	}
	function getFullVersion()
	{
		return $this->release.$this->status.$this->level;
	}
}