<?php
/**
 * Categories Model for JMyLife Component
 *
 * @package    JMyLife
 * @subpackage Components
 * @version    1.0.16
 * @copyright  2009-2010 Jeff Channell
 * components/com_jmylife/models/categories.php
 * @license

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 * @since      1.0.11
 */
// No direct access
defined( '_JEXEC' ) or die( 'Restricted access' );
// imports
jimport( 'joomla.application.component.model' );

/**
 * Categories Model
 *
 * @package    JMyLife
 * @subpackage Components
 */
class JmylifeModelCategories extends JModel
{
	/**
	 * data
	 * @var stdClass
	 */
	var $_data = null;

	/**
	 * Items total - for pagination
	 * @var integer
	 */
	var $_total = null;

	/**
	 * Pagination object
	 * @var object
	 */
	var $_pagination = null;

	/**
	 * contructor
	 */
	function __construct()
	{
		parent::__construct();

		$_JMLConfig = JmylifeConfiguration::getInstance();
		// J!1.6 doesn't use mainframe
		$appl =& JFactory::getApplication();

		$document =& JFactory::getDocument();
		$document->setTitle( $_JMLConfig->get( 'jmylife_name' ) );
		$document->setDescription( $_JMLConfig->get( 'jmylife_desc' ) );

		// Get pagination request variables
		$limit = $_JMLConfig->getInt( 'page_limit' );
		$limitstart = JRequest::getVar( 'limitstart', 0, '', 'int' );

		// In case limit has been changed, adjust it
		$limitstart = ( $limit != 0 ? ( floor( $limitstart / $limit ) * $limit ) : 0 );

		$this->setState( 'limit', $limit );
		$this->setState( 'limitstart', $limitstart );
  }

	/**
	 * builds category query
	 * @return string
	 *
	 */
	function _buildQuery()
	{
		$query = 'SELECT DISTINCT '
		       . 'c.*, COUNT(s.`id`) AS count '
					 . 'FROM `#__jmylife_categories` AS c '
					 . 'LEFT JOIN `#__jmylife_stories` AS s'
					 . ' ON ( s.`catid` = c.`id` AND s.`published` = 1 ) '
					 . 'WHERE c.`published` = 1 '
					 . 'GROUP BY c.`id` '
					 . 'ORDER BY c.`ordering` ASC, c.`name` ASC '
		;
		// all done!
		return $query;
	}

	/**
	 * gets list (override for long word handling)
	 * @return object list
	 * @since 1.0.4
	 * @renamed 1.0.5
	 */
	function _jmlGetList( $query, $limitstart, $limit )
	{
		JLoader::import('components.com_jmylife.helpers.html', JPATH_ADMINISTRATOR, '');
		$html = JmylifeHtmlHelper::getInstance();
		$_JMLConfig = JmylifeConfiguration::getInstance();
		$list = parent::_getList( $query, $limitstart, $limit );
		if( count( $list ) )
		{
			JLoader::import('components.com_jmylife.helpers.itemid', JPATH_ADMINISTRATOR, '');
			JLoader::import('components.com_jmylife.helpers.string', JPATH_ADMINISTRATOR, '');
			$itemid = JmylifeItemidHelper::getInstance();
			$string = JmylifeStringHelper::getInstance();
			foreach( $list as $key => $item )
			{
				$list[$key]->link = JRoute::_('index.php?option=com_jmylife&view=list&catid='.$list[$key]->id.'&Itemid='.$itemid->category( $list[$key]->id ) );
				$list[$key]->description = nl2br( $html->clean( $list[$key]->description ) );
				$list[$key]->name = $html->clean( $list[$key]->name );
				if( $_JMLConfig->getInt( 'break_long_words' ) == 1 )
				{
					$list[$key]->description = $string->breakLongWords( $list[$key]->description );
					$list[$key]->name = $string->breakLongWords( $list[$key]->name );
				}
				$style = '';
				if( strlen( $list[$key]->color ) > 0 || strlen( $list[$key]->background ) > 0 )
				{
					$style .= 'style="';
					if( strlen( $list[$key]->color ) > 0 )
						$style .= 'color: #'.$list[$key]->color.';';
					if( strlen( $list[$key]->background ) > 0 )
						$style .= 'background: #'.$list[$key]->background.';';
					$style .= '"';
				}
				$list[$key]->style = $style;
			}
		}
		return $list;
	}

	/**
	 * Gets the total amount of listings
	 * @return int
	 */
	function getTotal()
	{
		// Load the content if it doesn't already exist
		if( empty( $this->_total ) )
		{
			$query = $this->_buildQuery();
			$this->_total = $this->_getListCount( $query );
		}
		return $this->_total;
	}

	/**
	 * Gets the listings pagination
	 * @return JPagination object
	 */
	function getPagination()
	{
		// Load the content if it doesn't already exist
		if( empty( $this->_pagination ) )
		{
			jimport( 'joomla.html.pagination' );
			$this->_pagination = new JPagination( $this->getTotal(), $this->getState( 'limitstart' ), $this->getState( 'limit' ) );
		}
		return $this->_pagination;
	}

	/**
	 * Gets the categories as a list
	 * @return JPagination object
	 */
	function getCategories()
	{
		// Load the content if it doesn't already exist
		if( empty( $this->_categories ) )
		{
			$query = $this->_buildQuery();
			$this->_categories = $this->_jmlGetList( $query, $this->getState( 'limitstart' ), $this->getState( 'limit' ) );
		}
		return $this->_categories;
	}

	/**
	 * Gets the hidden inputs for the form
	 * @return string
	 */
	function getFields()
	{
		global $option, $view;
		// hack for now, fix later...
		$fields = '<input type="hidden" name="option" value="'.htmlentities($option).'" />'
		        . '<input type="hidden" name="view"   value="'.htmlentities($view).'" />'
		        ;
		return $fields;
	}

}