<?php
/**
 * List Model for JMyLife Component
 * 
 * @package    JMyLife
 * @subpackage Components
 * @version    1.0.16
 * @copyright  2009-2010 Jeff Channell
 * components/com_jmylife/models/list.php
 * @license

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 */
// No direct access
defined( '_JEXEC' ) or die( 'Restricted access' );
// imports
jimport( 'joomla.application.component.model' );
JLoader::import('components.com_jmylife.helpers.story', JPATH_ADMINISTRATOR, '');

/**
 * List Model
 *
 * @package    JMyLife
 * @subpackage Components
 */
class JmylifeModelList extends JModel
{
	/**
	 * data
	 * @var stdClass
	 */
	var $_data = null;

	/**
	 * Items total - for pagination
	 * @var integer
	 */
	var $_total = null;

	/**
	 * Pagination object
	 * @var object
	 */
	var $_pagination = null;

	/**
	 * category list
	 * @var array
	 */
	var $_categories = null;
	
	/**
	 * contructor
	 */
	function __construct()
	{
		parent::__construct();
		
		$_JMLConfig = JmylifeConfiguration::getInstance();
		// J!1.6 doesn't use mainframe
		$appl =& JFactory::getApplication();
		//$params	=& $appl->getParams( 'com_jmylife' );
		
		// prepare document since 1.0.8
		$catid = JRequest::getInt( 'catid', 0 );
		$document =& JFactory::getDocument();
		if( $catid > 0 )
		{
			$_JMLConfig->db->setQuery( 'SELECT `name`, `description` FROM `#__jmylife_categories` WHERE id = '.$catid );
			if( $_desc = $_JMLConfig->db->loadRow() )
			{
				$document->setTitle( $_desc[0] );
				if( strlen( $_desc[1] ) > 0 ) $document->setDescription( $_desc[1] );
			}
		}
		
		// Get pagination request variables
		$limit = $_JMLConfig->getInt( 'page_limit' );
		$limitstart = JRequest::getVar( 'limitstart', 0, '', 'int' );
		
		// In case limit has been changed, adjust it
		$limitstart = ( $limit != 0 ? ( floor( $limitstart / $limit ) * $limit ) : 0 );
		
		$this->setState( 'limit', $limit );
		$this->setState( 'limitstart', $limitstart );
  }

	/**
	 * builds category query
	 * @return string
	 * 
	 */
	function _buildCategoryQuery()
	{
		$query = 'SELECT * FROM `#__jmylife_categories` WHERE `published` = 1';
		// all done!
		return $query;
	}

	/**
	 * gets list (override for long word handling)
	 * @return object list
	 * @since 1.0.4
	 * @renamed 1.0.5
	 */
	function _jmlGetList( $query, $limitstart, $limit )
	{
		$list = parent::_getList( $query, $limitstart, $limit );
		$story = JmylifeStoryHelper::getInstance();
		return $story->prepareList( $list, 'list' );
	}

	/**
	 * Gets the list data
	 * @return stdClass object of listings
	 * 
	 * @todo use Joomla Tables instead of simple queries
	 */
	function getData()
	{
		// if data hasn't already been obtained, load it
		if( empty( $this->_data ) )
		{
			$story = JmylifeStoryHelper::getInstance();
			$query = $story->buildQuery( JRequest::get() );
			$this->_data = $this->_jmlGetList( $query, $this->getState( 'limitstart' ), $this->getState( 'limit' ) );
		}
		return $this->_data;
	}
	
	/**
	 * Gets the total amount of listings
	 * @return int
	 */
	function getTotal()
	{
		// Load the content if it doesn't already exist
		if( empty( $this->_total ) )
		{
			$story = JmylifeStoryHelper::getInstance();
			$query = $story->buildQuery(JRequest::get(), 1, 0, 1);
			$this->_total = $this->_getListCount( $query );
		}
		return $this->_total;
	}
	
	/**
	 * Gets the listings pagination
	 * @return JPagination object
	 */
	function getPagination()
	{
		// Load the content if it doesn't already exist
		if( empty( $this->_pagination ) )
		{
			jimport( 'joomla.html.pagination' );
			$this->_pagination = new JPagination( $this->getTotal(), $this->getState( 'limitstart' ), $this->getState( 'limit' ) );
		}
		return $this->_pagination;
	}
	
	/**
	 * Gets the categories as a list
	 * @return JPagination object
	 */
	function getCategories()
	{
		// Load the content if it doesn't already exist
		if( empty( $this->_categories ) )
		{
			$query = $this->_buildCategoryQuery();
			$this->_categories = $this->_getList( $query, 0, 99999 );
		}
		return $this->_categories;
	}
	
	/**
	 * Gets the hidden inputs for the form
	 * @return string
	 */
	function getFields()
	{
		global $option, $view;
		// hack for now, fix later...
		$fields = '<input type="hidden" name="option" value="'.htmlentities($option).'" />'
		        . '<input type="hidden" name="view"   value="'.htmlentities($view).'" />'
		        ;
		return $fields;
	}
}